<?php

class Speedy_Speedyshipping_Block_Adminhtml_System_Config_Form_Requirements extends Mage_Adminhtml_Block_System_Config_Form_Field
{
    private $mysqlVersion;

    public function _construct()
    {
        parent::_construct();

        $this->setTemplate('speedy_speedyshipping/system/config/info_table.phtml');
    }

    public function getColums()
    {
        return array(
            'REQUIREMENTS' => $this->__("Requirement"),
            'MIN'          => $this->__("MIN"),
            'MAX'          => $this->__("MAX"),
            'CURRENT'      => $this->__("Current"),
        );
    }

    public function getRows()
    {
        return array(
        'php_version' => array(
                'row' => array(
                    'REQUIREMENTS' => $this->__("PHP Version"),
                    'MIN'          => '5.6.x',
                    'MAX'          => '~',
                    'CURRENT'      => $this->getPhpVersion(),
                ),
                'warning' => array(
                    'CURRENT' => $this->isPhpCompatible('5.6')
                )
            ),
        'mysql' => array(
                'row' => array(
                    'REQUIREMENTS' => $this->__("MySQL Version"),
                    'MIN'          => '5.0.x',
                    'MAX'          => '~',
                    'CURRENT'      => $this->getMysqlVersion(),
                ),
                'warning' => array(
                    'CURRENT' => $this->isMysqlCompatible('5.0')
                )
            ),
        'soap' => array(
                'row' => array(
                    'REQUIREMENTS' => $this->__("SOAP"),
                    'MIN'          => '~',
                    'MAX'          => '~',
                    'CURRENT'      => $this->isSoapCompatible() ? $this->__("FULFILLED") : $this->__("NOT FULFILLED"),
                ),
                'warning' => array(
                    'CURRENT' => !$this->isSoapCompatible()
                )
            ),
        );
    }

    private function isPhpCompatible($min)
    {
        $php_version = $this->getPhpVersion();

        return version_compare($php_version, $min, '<');
    }

    private function getPhpVersion()
    {
        return preg_replace('/^([0-9\.]+).*/', '$1', phpversion());
    }

    private function isMysqlCompatible($min)
    {
        $mysql_version = $this->getMysqlVersion();

        return version_compare($mysql_version, $min, '<');
    }

    private function getMysqlVersion()
    {
        if (empty($this->mysqlVersion)) {
            $conn = Mage::getSingleton('core/resource')->getConnection('core_read');

            $mysql_version = $conn->fetchOne("SELECT VERSION() as mysql_version");

            if (!empty($mysql_version)) {
                $this->mysqlVersion = preg_replace('/^([0-9\.]+).*/', '$1', $mysql_version);
            }
        }

        return $this->mysqlVersion;
    }

    private function isSoapCompatible()
    {
        return class_exists('SOAPClient');
    }

    protected function _getElementHtml(Varien_Data_Form_Element_Abstract $element)
    {
        return $this->_toHtml();
    }
}
